@@ -1,29 +1,13 @@
-# $Id: Reader.pm,v 1.1.2.1 2004/04/20 20:09:48 pajas Exp $
-#
-# This is free software, you may use it and distribute it under the same terms as
-# Perl itself.
-#
-# Copyright 2001-2003 AxKit.com Ltd., 2002-2006 Christian Glahn, 2006-2009 Petr Pajas
-#
-#
 package XML::LibXML::Reader;
-
 use XML::LibXML;
-use Carp;
+
 use strict;
 use warnings;
 
 use vars qw ($VERSION);
-$VERSION = "1.69_2"; # VERSION TEMPLATE: DO NOT CHANGE
-
-use 5.008_000;
-
-BEGIN {
-  UNIVERSAL::can('XML::LibXML::Reader','_newForFile') or
-      croak("Cannot use XML::LibXML::Reader module - ".
-	    "your libxml2 is compiled without reader support!");
-}
+$VERSION = "1.62"; # VERSION TEMPLATE: DO NOT CHANGE
 
+use Carp;
 use base qw(Exporter);
 use constant {
     XML_READER_TYPE_NONE => 0,
@@ -56,8 +40,6 @@ use constant {
 };
 use vars qw( @EXPORT @EXPORT_OK %EXPORT_TAGS );
 
-sub CLONE_SKIP { 1 }
-
 BEGIN {
 
 %EXPORT_TAGS = (
@@ -235,14 +217,5 @@ sub preservePattern {
   }
 }
 
-sub nodePath {
-  my $reader=shift;
-  my $path = $reader->_nodePath;
-  $path=~s/\[\d+\]//g; # make /foo[1]/bar[1] just /foo/bar, since
-                       # sibling count in the buffered fragment is
-                       # basically random and generally misleading
-  return $path;
-}
-
 1;
 __END__
