@@ -1,11 +1,3 @@
-/* $Id$
- *
- * This is free software, you may use it and distribute it under the same terms as
- * Perl itself.
- *
- * Copyright 2001-2003 AxKit.com Ltd., 2002-2006 Christian Glahn, 2006-2009 Petr Pajas
-*/
-
 #include <libxml/tree.h>
 #include <libxml/xpath.h>
 #include <libxml/xpathInternals.h>
@@ -14,7 +6,6 @@
 #include "EXTERN.h"
 
 #include "dom.h"
-#include "xpath.h"
 
 void
 perlDocumentFunction(xmlXPathParserContextPtr ctxt, int nargs){
@@ -150,28 +141,17 @@ perlDocumentFunction(xmlXPathParserContextPtr ctxt, int nargs){
  **/
 
 xmlXPathObjectPtr
-domXPathFind( xmlNodePtr refNode, xmlChar * path, int to_bool ) {
-    xmlXPathObjectPtr res = NULL;
-    xmlXPathCompExprPtr comp;
-    comp = xmlXPathCompile( path );
-    if ( comp == NULL ) {
-        return NULL;
-    }
-    res = domXPathCompFind(refNode,comp,to_bool);
-    xmlXPathFreeCompExpr(comp);
-    return res;
-}
-
-xmlXPathObjectPtr
-domXPathCompFind( xmlNodePtr refNode, xmlXPathCompExprPtr comp, int to_bool ) {
+domXPathFind( xmlNodePtr refNode, xmlChar * path ) {
     xmlXPathObjectPtr res = NULL;
   
-    if ( refNode != NULL && comp != NULL ) {
+    if ( refNode != NULL && path != NULL ) {
         xmlXPathContextPtr ctxt;
+        xmlXPathCompExprPtr comp;
 
         xmlDocPtr tdoc = NULL;
         xmlNodePtr froot = refNode;
 
+        comp = xmlXPathCompile( path );
         if ( comp == NULL ) {
             return NULL;
         }
@@ -192,9 +172,8 @@ domXPathCompFind( xmlNodePtr refNode, xmlXPathCompExprPtr comp, int to_bool ) {
                 froot = froot->parent;
             }
             xmlAddChild((xmlNodePtr)tdoc, froot);
-            xmlSetTreeDoc(froot, tdoc); /* probably no need to clean psvi */
-	    froot->doc = tdoc;
-            /* refNode->doc = tdoc; */
+
+            refNode->doc = tdoc;
         }
 
         /* prepare the xpath context */
@@ -217,21 +196,11 @@ domXPathCompFind( xmlNodePtr refNode, xmlXPathCompExprPtr comp, int to_bool ) {
         xmlXPathRegisterFunc(ctxt,
                              (const xmlChar *) "document",
                              perlDocumentFunction);
-	if (to_bool) {
-#if LIBXML_VERSION >= 20627
-	  int val = xmlXPathCompiledEvalToBoolean(comp, ctxt);
-	  res = xmlXPathNewBoolean(val);
-#else
-	  res = xmlXPathCompiledEval(comp, ctxt);
-	  if (res!=NULL) {
-	    int val = xmlXPathCastToBoolean(res);
-            xmlXPathFreeObject(res);
-	    res = xmlXPathNewBoolean(val);
-	  }
-#endif
-	} else {
-	  res = xmlXPathCompiledEval(comp, ctxt);
-	}
+       
+        res = xmlXPathCompiledEval(comp, ctxt);
+
+        xmlXPathFreeCompExpr(comp);
+
         if (ctxt->namespaces != NULL) {
             xmlFree( ctxt->namespaces );
         }
@@ -241,9 +210,7 @@ domXPathCompFind( xmlNodePtr refNode, xmlXPathCompExprPtr comp, int to_bool ) {
         if ( tdoc != NULL ) {
             /* after looking through a fragment, we need to drop the
                fake document again */
-            xmlSetTreeDoc(froot, NULL); /* probably no need to clean psvi */
-	    froot->doc = NULL;
-	    froot->parent = NULL;
+            xmlSetTreeDoc(froot, NULL);
             tdoc->children = NULL;
             tdoc->last     = NULL;
             /* next line is not required anymore */
@@ -255,36 +222,12 @@ domXPathCompFind( xmlNodePtr refNode, xmlXPathCompExprPtr comp, int to_bool ) {
     return res;
 }
 
-/* this function is not actually used: */
 xmlNodeSetPtr
 domXPathSelect( xmlNodePtr refNode, xmlChar * path ) {
     xmlNodeSetPtr rv = NULL;
     xmlXPathObjectPtr res = NULL;
   
-    res = domXPathFind( refNode, path, 0 );
-    
-    if (res != NULL) {
-            /* here we have to transfer the result from the internal
-               structure to the return value */
-        	/* get the result from the query */
-        	/* we have to unbind the nodelist, so free object can 
-        	   not kill it */
-        rv = res->nodesetval;  
-        res->nodesetval = 0 ;
-    }
-
-    xmlXPathFreeObject(res);
-
-    return rv;
-}
-
-/* this function is not actually used: */
-xmlNodeSetPtr
-domXPathCompSelect( xmlNodePtr refNode, xmlXPathCompExprPtr comp ) {
-    xmlNodeSetPtr rv = NULL;
-    xmlXPathObjectPtr res = NULL;
-  
-    res = domXPathCompFind( refNode, comp, 0 );
+    res = domXPathFind( refNode, path );
     
     if (res != NULL) {
             /* here we have to transfer the result from the internal
@@ -309,34 +252,26 @@ domXPathCompSelect( xmlNodePtr refNode, xmlXPathCompExprPtr comp ) {
  **/
 
 xmlXPathObjectPtr
-domXPathFindCtxt( xmlXPathContextPtr ctxt, xmlChar * path, int to_bool ) {
+domXPathFindCtxt( xmlXPathContextPtr ctxt, xmlChar * path ) {
     xmlXPathObjectPtr res = NULL;
+  
     if ( ctxt->node != NULL && path != NULL ) {
         xmlXPathCompExprPtr comp;
+
+        xmlDocPtr tdoc = NULL;
+        xmlNodePtr froot = ctxt->node;
+
         comp = xmlXPathCompile( path );
         if ( comp == NULL ) {
             return NULL;
         }
-        res = domXPathCompFindCtxt(ctxt,comp,to_bool);
-        xmlXPathFreeCompExpr(comp);
-    }
-    return res;
-}
-
-xmlXPathObjectPtr
-domXPathCompFindCtxt( xmlXPathContextPtr ctxt, xmlXPathCompExprPtr comp, int to_bool ) {
-    xmlXPathObjectPtr res = NULL;
-    if ( comp != NULL && ctxt->node != NULL && comp != NULL ) {
-        xmlDocPtr tdoc = NULL;
-        xmlNodePtr froot = ctxt->node;
-
+        
         if ( ctxt->node->doc == NULL ) {
             /* if one XPaths a node from a fragment, libxml2 will
                refuse the lookup. this is not very usefull for XML
                scripters. thus we need to create a temporary document
                to make libxml2 do it's job correctly.
              */
-
             tdoc = xmlNewDoc( NULL );
 
             /* find refnode's root node */
@@ -347,36 +282,24 @@ domXPathCompFindCtxt( xmlXPathContextPtr ctxt, xmlXPathCompExprPtr comp, int to_
                 froot = froot->parent;
             }
             xmlAddChild((xmlNodePtr)tdoc, froot);
-	    xmlSetTreeDoc(froot,tdoc);  /* probably no need to clean psvi */
-            froot->doc = tdoc;
-	    /* ctxt->node->doc = tdoc; */
+
+            ctxt->node->doc = tdoc;
         }
-	if (to_bool) {
-#if LIBXML_VERSION >= 20627
-	  int val = xmlXPathCompiledEvalToBoolean(comp, ctxt);
-	  res = xmlXPathNewBoolean(val);
-#else
-	  res = xmlXPathCompiledEval(comp, ctxt);
-	  if (res!=NULL) {
-	    int val = xmlXPathCastToBoolean(res);
-            xmlXPathFreeObject(res);
-	    res = xmlXPathNewBoolean(val);
-	  }
-#endif
-	} else {
-	  res = xmlXPathCompiledEval(comp, ctxt);
-	}
+       
+        res = xmlXPathCompiledEval(comp, ctxt);
+
+        xmlXPathFreeCompExpr(comp);
+
         if ( tdoc != NULL ) {
             /* after looking through a fragment, we need to drop the
                fake document again */
-	    xmlSetTreeDoc(froot,NULL); /* probably no need to clean psvi */
+	    xmlSetTreeDoc(froot,NULL);
             froot->doc = NULL;
-            froot->parent  = NULL;
             tdoc->children = NULL;
             tdoc->last     = NULL;
-	    if (ctxt->node) {
-	      ctxt->node->doc = NULL;
-	    }
+            froot->parent  = NULL;
+            ctxt->node->doc = NULL;
+
             xmlFreeDoc( tdoc );
         }
     }
@@ -388,7 +311,7 @@ domXPathSelectCtxt( xmlXPathContextPtr ctxt, xmlChar * path ) {
     xmlNodeSetPtr rv = NULL;
     xmlXPathObjectPtr res = NULL;
   
-    res = domXPathFindCtxt( ctxt, path, 0 );
+    res = domXPathFindCtxt( ctxt, path );
     
     if (res != NULL) {
             /* here we have to transfer the result from the internal
